# $Id: Server.pm,v 1.1 2005/06/17 15:45:56 mike Exp $

# Diagnostic codes are listed at
# http://www.openarchives.org/OAI/openarchivesprotocol.html#ErrorConditions
#
# ### Could add gzip encoding support as in:
#	HTTP-OAI-3.08/bin/oai_static_gateway.pl

package Alvis::OAI::Server;
use strict;
use warnings;

use HTTP::Daemon;
use HTTP::Status;
use CGI;
use HTTP::OAI;
use XML::SAX::Writer;
use Alvis::Logger;


sub new {
    my $class = shift();
    my($port, $repository, $loglevel) = @_;
    die "$class: no port specified" if !defined $port;
    die "$class: no repository specified" if !defined $repository;

    my $server = new HTTP::Daemon(LocalPort => $port, ReuseAddr => 1)
	or die "$class: can't create HTTP::Daemon: $!";
    $repository->baseURL($server->url());

    return bless {
	server => $server,
	repository => $repository,
	logger => new Alvis::Logger(level => $loglevel),
    }, $class;
}


sub log { my $this = shift(); $this->{logger}->log(@_) }


#   ###	There is a bug here: because of the inner loop on multiple
#	requests from a single connection, we do not accept additional
#	connections until the first one has closed.  We could fix this
#	by refusing keepalives, by forking for each connection, or by
#	select()ing on the listener and the connections.
#
sub serve {
    my $this = shift();
    my $server = $this->{server};
    my $repository = $this->{repository};

    $this->log(1, "listening");
    while (my $c = $server->accept()) {
	$this->log(2, "new connection $c");
	while (my $request = $c->get_request()) {
	    if ($request->method ne 'GET' && $request->method ne 'POST') {
		$c->send_error(RC_FORBIDDEN);
	    } elsif ($request->uri eq "/favicon.ico") {
		# Browsers ask for this all the time
		$c->send_error(RC_NOT_FOUND);
	    } else {
		# We could insist on a particular path here
		$this->_handle_request($c, $request, $repository);
	    }
	}
	my $reason = $c->reason();
	$this->log(2, ($reason eq "Client closed" || $reason eq "")
	       ? "connection $c closed" :
	       "couldn't read request: No more requests from this connection");
	$c->close();
    }
    $this->log(0, "server->accept() failed: closing down");
}


sub _handle_request {
    my $this = shift();
    my($c, $request, $repository) = @_;

    my $path = $request->uri;
    $this->log(3, $request->method, " ", $path);
    my $query = $path;
    $query =~ s/.*?\?//;

    my $cgi = new CGI($query);
    $this->log(3, "query parameters: ",
	   join(", ", map { "$_='" . $cgi->param($_) . "'" } $cgi->param()));
    my $response = undef;

    my %vars = $cgi->Vars();
    my @errs = HTTP::OAI::Repository::validate_request(%vars);
    if (!@errs) {
	my $verb = delete $vars{'verb'};
	my($code, $message);
	($response, $code, $message) = $repository->handle($verb, %vars);
	if (!defined $response) {
	    die "handle() failed but returned no error-code" if !defined $code;
	    push @errs, new HTTP::OAI::Error(code => $code,
					     message => $message);
	}
    }

    # For some reason, we have to set requestURL ourselves: something
    # that the module providing the service really ought to do.  The
    # sample code in the HTTP::OAI documentation uses the CGI module's
    # self_url(), but that yields a bad path build using $0, which in
    # general has nothing to do with the request the UA made, so we
    # build it out of bits of the request object.  The so-called uri()
    # method does _not_ yield a URI as advertised, only a
    # host-specific path.
    my $requestURL = "http://" . $request->header("host") . $request->uri;

    if (defined $response) {
	$response->requestURL($requestURL)
	    if ref $response;
    } else {
	$this->log(3, scalar(@errs), " error", (@errs == 1 ? "" : "s"), ": ",
	       join("; ", map { $_->code . " (" . $_->message . ")" } @errs));
	die "assertion failed: response undefined and no errors set"
	    if !@errs;
	# Build a response object that contains nothing but error messages
	$response = HTTP::OAI::Response->new(requestURL => $requestURL);
	$response->errors(@errs);
    }

    ### Why doesn't $c->send_response($response) work?
    #	Can't call method "header" on an undefined value at (eval 9) line 1.
    my $text;
    if (ref $response) {
	# "Proper" mechanism: back-end generated an HTTP::OAI::Response
	$response->set_handler(XML::SAX::Writer->new(Output=>\$text));
	$response->generate();
    } else {
	# Hacked case, since the proper one is so opaque: just a string
	$text = $response;
    }
    $this->_send_content($c, "text/xml", $text);
}


### This should be a method of HTTP::Daemon::ClientConn
sub _send_content {
    my $this = shift();
    my($c, $type, $text) = @_;

    $c->send_basic_header();
    # Sending content-length allows connection re-use
    $c->print("Content-type: $type");
    $c->send_crlf();
    $c->print("Content-length: ", length($text));
    $c->send_crlf();
    $c->send_crlf();
    $c->print($text);
    $this->log(4, "sent response:\n$text");
}


1;
